function [mfit,es,une] = evaluate(P,GAP,cne,D)
% EVALUATE  Evaluates the fitness of a population of chromosomes.
%           Note that this routine only updates those memebers of
%           the population whos fitness has not already been evaluated
%           Observe that all genes of a populaton member are passed, as well
%           as the region number for that member of the population
%
% [mfit,es,une] = evaluate(P,GAP,cne,[D])
%
% Inputs:
% P        = current population of chromosomes (a structure)
% GAP      = a structure with the genetic algorith parameters
% cne      = current number of evaluations performed thus far
% D        = an optional argument or structure with all data needed to evaluate the function
%
% Outputs:
% mfit     = multiobjective fitness (number of objectives by size of population)
% es       = evaluation status of each member of population
% une      = updated number of evaluations
%
% Written by:
% S.D. Sudhoff
% School of Electrical and Computer Engineering
% 1285 Electrical Engineering Building
% West Lafayette, IN 47907-1285
% Phone: 765-494-3246
% Fax: 765-494-0676
% E-mail: sudhoff@ecn.purdue.edu

% check on the number of input argument
if ~((nargin==3)|(nargin==4))
   error('Error in Evaluate: Wrong number of input arguments');
end

% determine if the extra data needs to be passed to the evaluation routine 
extra_data_present=(nargin==4);
if extra_data_present
   extra_data_present=size(D,1)>0;
end

% initialize the fitness matrix
mfit=P.mfit;

% find the elements whos indices need to be updated
if GAP.ev_are
   update_indices=1:P.size;
else
   update_indices=find(P.eval==0);
end

% find the number of updates to make  
nupdates=length(update_indices);

% see if there are any updates to make
if nupdates>0

   % initialize the gene matrix if using a block evaluation
   if GAP.ev_bev
      
      % block evaluation
      if extra_data_present
         if GAP.ev_ssd 
            mfit(:,update_indices)=feval(P.fithandle, ...
                                         P.gene(:,update_indices), ...
                                         P.age(update_indices), ...
                                         P.mfit(:,update_indices), ...
                                         P.region(:,update_indices),D);
         else
            mfit(:,update_indices)=feval(P.fithandle,P.gene(:,update_indices),D);
         end
      else
         if GAP.ev_ssd
            mfit(:,update_indices)=feval(P.fithandle, ...
                                         P.gene(:,update_indices), ...
                                         P.age(update_indices), ...
                                         P.mfit(:,update_indices), ...
                                         P.region(:,update_indices));
         else
            mfit(:,update_indices)=feval(P.fithandle,P.gene(:,update_indices));
         end	
      end    
         
   else

      % single evaluation
      if extra_data_present
         if GAP.ev_ssd 
            for i=1:nupdates
               j=update_indices(i);
               mfit(:,j)=feval(P.fithandle,P.gene(:,j),P.age(j),P.mfit(:,j),P.region(j),D);
            end
         else
            for i=1:nupdates
               j=update_indices(i);
               mfit(:,j)=feval(P.fithandle,P.gene(:,j),D);
            end
         end
      else
         if GAP.ev_ssd
            for i=1:nupdates
                j=update_indices(i);
                mfit(:,j)=feval(P.fithandle,P.gene(:,j),P.age(j),P.mfit(:,j),P.region(j));
            end
         else
            for i=1:nupdates
                j=update_indices(i);
                mfit(:,j)=feval(P.fithandle,P.gene(:,j));
            end
         end	
      end    
  
   end
      
end % updates > 0

% update evaluation status (since everything is now evaluated)
es=ones(1,size(mfit,2));

% statistics
une=cne+nupdates;

%  Copyright 2010 - Scott Sudhoff 
% 
%  The program is distributed under the terms of the GNU Lesser
%  General Public License(GNU LGPL). 
% 
%  This file is part of GOSET
% 
%  GOSET is free software: you can redistribute it and/or modify
%  it under the terms of the GNU Lesser General Public License as published
%  by the Free Software Foundation, either version 3 of the License, or 
%  (at your option) any later version.
% 
%  GOSET is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU Lesser General Public License for more details.
% 
%  You should have received a copy of the GNU Lesser General Public
%  License along with GOSET.  If not, see <http://www.gnu.org/licenses/>.
