function [fP,GAS,bI,f]= gaoptimize(objhandle,GAPic,D,GAS,iP,GUIhdl)

% GAOPTIMIZE  Optimizes a function using a genetic algorithm.
%
% [fP,GAS]      = gaoptimize(@fitfun,GAP,D,GAS,iP,GUIhd1)
% [fP,GAS]      = gaoptimize(@fitfun,GAP,D,GAS,iP)
% [fP,GAS]      = gaoptimize(@fitfun,GAP,D)
% [fP,GAS]      = gaoptimize(@fitfun,GAP)
% [fP,GAS,bI]   = gaoptimize(@fitfun,GAP,D,GAS,iP,GUIhd1)
% [fP,GAS,bI]   = gaoptimize(@fitnun,GAP,D,GAS,iP)
% [fP,GAS,bI]   = gaoptimize(@fitnun,GAP,D)
% [fP,GAS,bI]   = gaoptimize(@fitnun,GAP)
% [fP,GAS,bI,f] = gaoptimize(@fitfun,GAP,D,GAS,iP,GUIhd1)
% [fP,GAS,bI,f] = gaoptimize(@fitnun,GAP,D,GAS,iP)
% [fP,GAS,bI,f] = gaoptimize(@fitnun,GAP,D)
% [fP,GAS,bI,f] = gaoptimize(@fitnun,GAP)
%
% Inputs:
% fitfun   = name of *.m function that evaluates the fitness.  If blckeval is set to zero
%            (see below) the input is a vector of length equal to the number of genes,
%            and the output is a column vector of fitness functions of length equal
%            to the number of objectives. If blckeval is set to 1 for block evaluation
%            the input is a matrix where the each row corresponds to a different gene
%            and each column is a different individual.  In this case the output is
%            also a matrix where each row corresponds to a different objective and each
%            column is a different member of the population.
% GAP      = Genetic Algorithm Parameters
%            (see gap_default for definitions)
% D        = data needed by fitness function (this could be an empty matrix)
% GAS      = Genetic Algorithm Statistics (Use empty matrix when not in use)
% iP       = Optional variable with intial population (Use empty matrix when not in use)
% GUIhdl   = Handle for Graphical User Interface (Use empty matrix when not in use)
%
% Outputs:
% bI       = Best individual (For single objective case, these are the gene
%            values for the best individual. For the multi objective case,
%            each column is a non-dominated individual(sorted according to the specified 
%            objective).
% fP       = Final population
% GAS      = Genetic Algorithm Statistics
% f        = Best fitness/fitnesses(Each column is a set of fitness values
%            of the individuals corresponding to bI)
%
% Written by:
% S.D. Sudhoff
% School of Electrical and Computer Engineering
% 1285 Electrical Engineering Building
% West Lafayette, IN 47907-1285
% Phone: 765-494-3246
% Fax: 765-494-0676
% E-mail: sudhoff@ecn.purdue.edu

global simflag
simflag = 1;

% alternate input form for GAP
if isfield(GAPic,'gd')
    GAPic.gd_min=GAPic.gd(:,1)';
    GAPic.gd_max=GAPic.gd(:,2)';
    GAPic.gd_type=GAPic.gd(:,3)';
    GAPic.gd_cid=GAPic.gd(:,4)';
end

% initialize GAP
GAP=GAPic;

% set up
switch nargin
    case 6
    case 5
        GUIhdl=[];
    case 3
        GAS=[];
        iP=[];
        GUIhdl=[];
    case 2
        D=[];
        GAS=[];
        iP=[];
        GUIhdl=[];
    otherwise
        error('Incorrect number of arguments to gaoptimize');
end

switch nargout
    case 2
        bI=[];
    case 3
        f=[];
    case 4
    otherwise
        error('Incorrect number of output arguments to gaoptimize');
end

% initialization
[GAP,GAS,P] = gainit(objhandle,D,GAP,GAS,iP,GUIhdl);
initgen = GAS.cg;

% let the population evolve
for j=initgen:GAP.fp_ngen,
    
    GAS.cg = j;                                                   % Update current population number
    GAP=gapadjust(GAPic,GAS);                                     % Update GA Parameters
    
    GAP.owv                     = objwght(GAP);                   % Objective Weighting Vector for Current Generation
    P.pen                       = divcon(P,GAP);                  % Diversity Control
    P.fit                       = scale(P,GAP);                   % Determine the Scaled Fitness
    [P_list,PL_size]            = select(P,GAP);                  % Select Mating Pool / Parent List
    D_list                      = death(P,P_list,GAP);            % Select Individuals to Die
    P2                          = matingcrossover(P, ...          % Mating and Crossover
        P_list,PL_size,D_list,GAP,GAS);
    P3                          = mutate(P2,GAP);                 % Mutation
    P4                          = migrate(P3,GAP,GAS.cg);         % Migration
    P4.age                      = updateage(P4);                  % Update age
    [P4.mfit,P4.eval,GAS.ne(j)] = evaluate(P4,GAP,GAS.ne(j-1),D); % Evaluate
    P5                          = elitism(P4,P,GAP,GAS);          % Elitism
    [P,GAS]                     = randsearch(P5,GAP,GAS,D);       % Random Search
    [GAS] = updatestat(GAP,GAS,P);                                % Update statistics
    reportplot(GAP,GAS,P,GUIhdl);                                 % Update report plot
    
    if simflag == 0  break; end                                   % STOP flag check
    
end

fP = P;      % Final population

% Updated 1 July 07
% Describe simulation
if (GAP.fp_obj~=0)
    disp(['This single-objective optimization of objective ' ...
        num2str(GAP.fp_obj) ' using a population size of ' ...
        num2str(GAP.fp_npop) ' over ' num2str(GAP.fp_ngen) ' generations']);
else
    disp(['This multi-objective optimization ' ...
        'used a population size of ' num2str(GAP.fp_npop) ' over ' ...
        num2str(GAP.fp_ngen) ' generations']);
end

% Updated 1 July 07
% Pick out and return best individual
if (nargout==3)||(nargout==4)
    if (GAP.fp_obj>0)
        % third line was in error - fixed 6/22/08
        if GAP.trimga
            disp('Trimming best answer ...');
            if nargin>2
                [bI,f]=trimga(GAP,fP,D);
            else
                [bI,f]=trimga(GAP,fP);
            end
            bI=bI';
        else
            bI=GAS.bestgenes(:,end,GAP.fp_obj);
        end
    else
        %       nd=nondom(P.mfit,1);
        %       ndi=find(nd==1);
        %       bI=fP.gene(:,ndi);
        % Adding the Pareto plot code.
        mat1 = P.mfit;
        ind1 = nondom(mat1,1); % Matrix containing pointers to non dominated solutions.
        ind2 = find(ind1);
        mat1_nd = mat1(:,ind2);
        opt_val = 1./mat1_nd;
        gene_val = P.gene(:,ind2);
        
        % GAP.fp_nobj is the number of objectives.
        if GAP.fp_nobj < GAP.so_ob
            error('Wrong input to GAP.so_ob')
        end
        %
        % Sorting based on user input.
        [dummy1,ind3] = sort(opt_val(GAP.so_ob,:));
        obj_srt = sortrows(opt_val',GAP.so_ob);
        [nsols,nobjs] = size(obj_srt);
        % Each column of obj_srt represents each objective sorted according to the
        % specified objective
        gene_srt = gene_val(:,ind3);
        for ind4 = 1:GAP.fp_nobj
            figure
            plot(1:nsols,obj_srt(:,ind4),'xb')
            xlabel('Design number')
            l1 = num2str(ind4);
            ylabel(['Objective ',l1])
        end
        bI = gene_srt; % bI is a matrix with all the non-dominated solutions' gene values
        f = obj_srt'; % f is the objective values 
    end
end
if GAP.fp_nobj == 2
    figure
    plot(obj_srt(:,1),obj_srt(:,2),'xb');
% Labeling
xlabel('Objective 1');
ylabel('Objective 2');
title('Pareto Optimal Front') ;
end

%  Copyright 2010 - Scott Sudhoff 
% 
%  The program is distributed under the terms of the GNU Lesser
%  General Public License(GNU LGPL). 
% 
%  This file is part of GOSET
% 
%  GOSET is free software: you can redistribute it and/or modify
%  it under the terms of the GNU Lesser General Public License as published
%  by the Free Software Foundation, either version 3 of the License, or 
%  (at your option) any later version.
% 
%  GOSET is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU Lesser General Public License for more details.
% 
%  You should have received a copy of the GNU Lesser General Public
%  License along with GOSET.  If not, see <http://www.gnu.org/licenses/>.
